@@ -32,6 +32,10 @@ class PreprocessHandler:
     function: Callable[[], List[PreprocessResponse]]
     data_length: Dict[DataStateType, int] = field(default_factory=dict)
 
+@dataclass
+class UnlabeledDataPreprocessHandler:
+    function: Callable[[], PreprocessResponse]
+    data_length: int = 0
 
 VisualizerCallableInterface = Union[
     Callable[..., LeapImage],
@@ -97,6 +101,7 @@ class PredictionTypeHandler:
 @dataclass
 class DatasetIntegrationSetup:
     preprocess: Optional[PreprocessHandler] = None
+    unlabeled_data_preprocess: Optional[UnlabeledDataPreprocessHandler] = None
     visualizers: List[VisualizerHandler] = field(default_factory=list)
     inputs: List[InputHandler] = field(default_factory=list)
     ground_truths: List[GroundTruthHandler] = field(default_factory=list)
@@ -108,7 +113,7 @@ class DatasetIntegrationSetup:
 @dataclass
 class DatasetSample:
     inputs: Dict[str, npt.NDArray[np.float32]]
-    gt: Dict[str, npt.NDArray[np.float32]]
+    gt: Optional[Dict[str, npt.NDArray[np.float32]]]
     metadata: Dict[str, Union[str, int, bool, float]]
     index: int
     state: DataStateEnum
